import { selectSource } from './common';

interface HarvesterMemory extends CreepMemory {
  sourceId: Id<Source> | undefined;
}

export function runAsHarvester(creep: Creep) {
  const memory = creep.memory as HarvesterMemory;

  if (memory.sourceId === undefined) {
    memory.sourceId = selectSource(creep);
  }

  if (creep.store.getFreeCapacity() > 0) {
    const source = Game.getObjectById(memory.sourceId);
    if (source && creep.harvest(source) === ERR_NOT_IN_RANGE) {
      creep.say('🔄 harvest');
      creep.moveTo(source, { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  } else {
    let targets = creep.room.find(FIND_STRUCTURES, {
      filter: (structure) => {
        return (
          (structure.structureType === STRUCTURE_EXTENSION ||
            structure.structureType === STRUCTURE_SPAWN ||
            structure.structureType === STRUCTURE_TOWER) &&
          structure.store.getFreeCapacity(RESOURCE_ENERGY) > 0
        );
      }
    });

    if (targets.length > 0) {
      if (creep.transfer(targets[0], RESOURCE_ENERGY) === ERR_NOT_IN_RANGE) {
        creep.say('🚛 transfer');
        creep.moveTo(targets[0], { visualizePathStyle: { stroke: '#ffffff' } });
      }
    } else {
      const spawns = creep.room.find(FIND_MY_SPAWNS);
      if (spawns.length > 0) {
        creep.say('to spawn');
        creep.moveTo(spawns[0]);
      }
    }
  }
}
